/*-------------------------------------------------------------------------
 
    FILE....: auto_conf_dgv.c
    AUTHOR..: Luciano Alves Barroso - desenvolvimento@digivoice.com.br
    DATE....: 24/11/2006
 
    auto-configuration(digivoice.conf) for DigiVoice cards

-------------------------------------------------------------------------*/

#include <stdio.h>
#include <stdlib.h>
#ifdef __LINUX__
	#include <strings.h>
	#include <pthread.h>
	#include <getopt.h>
#else
	#include <string.h>
	#include <conio.h>
	#include <windows.h>
	#define DllImport   __declspec(dllimport)
#endif


#include <voicerlib/voicerlib.h>


//function prototypes
void CopyFileToFile(char *szSourceFile, char *szTargetFile);
void ConfigGroups(short nCard);
void ConfigPorts(short nCard);
void ConfigE1(int nCardsCount);

#define MAX_PATH 	255
#define MAX_CH		360

int nLastGroup;

void CopyFileToFile(char *szSourceFile, char *szTargetFile)
{
    FILE *SourceFile;
    FILE *TargetFile;    
    char szData[MAX_PATH];

    if ((SourceFile = fopen(szSourceFile, "r")) == NULL)
        printf("\n\nError Reading SourceFile");

    if (szSourceFile == "step_1.conf")
    {
		if ((TargetFile = fopen(szTargetFile, "w")) == NULL)
    	    printf("\n\nError Reading TargetFile");
	}
    else    
    {
		if ((TargetFile = fopen(szTargetFile, "a")) == NULL)
    	    printf("\n\nError Reading TargetFile");
    }
    
    while (fgets(szData, MAX_PATH - 1, SourceFile) != NULL)
    {
        if (fputs(szData, TargetFile) == -1)    
		    printf("\n\nError Writing TargetFile");
    }	

    //printf("Copy File '%s' To File '%s'...OK\n", szSourceFile, szTargetFile);    
    fclose(SourceFile);
    fclose(TargetFile); 
}

void ConfigGroups(short nCard)
{
    FILE *TargetFile;    
    int nCardInterface, nCardPortsCount, nFirstPort, nLastPort;

    nCardInterface = dg_GetCardInterface(nCard);
    nCardPortsCount = dg_GetCardPortsCount(nCard);
    nFirstPort = dg_GetAbsolutePortNumber(nCard, 1); 
    nLastPort = dg_GetAbsolutePortNumber(nCard, nCardPortsCount); 

    if ((TargetFile = fopen("digivoice.conf", "a")) == NULL)
    {
        printf("\n\nError Reading TargetFile-ConfigGroups");
    }

    switch(nCardInterface)
    {
		case DG_DIGITAL_INTERFACE:
		    printf("Card Interface - Ports: DIGITAL_INTERFACE - %d\n", nCardPortsCount);
	
		    if (nCardPortsCount == 60)
		    {	    
				nLastGroup++;
		
				//write group
	    	    printf("[groups]\n");
				printf("group = %d\n", nLastGroup);
		        printf("ports => %d-%d\n", nFirstPort, nLastPort - 30);
				printf("\n");
			
				fprintf(TargetFile, "group = %d\n", nLastGroup);
				fprintf(TargetFile, "ports => %d-%d\n", nFirstPort, nLastPort - 30);	        	
				fprintf(TargetFile, "\n");    	    
			
				nLastGroup++;
		
				//write group
	    	    printf("[groups]\n");
				printf("group = %d\n", nLastGroup);
		        printf("ports => %d-%d\n", nLastPort - 29, nLastPort);
				printf("\n");

				fprintf(TargetFile, "group = %d\n", nLastGroup);
				fprintf(TargetFile, "ports => %d-%d\n", nLastPort - 29, nLastPort);
				fprintf(TargetFile, "\n");
	    	}
	    	else
	    	{
				nLastGroup++;
	    
				//write group
				printf("[groups]\n");
				printf("group = %d\n", nLastGroup);
				printf("ports => %d-%d\n", nFirstPort, nLastPort);
				printf("\n");

				fprintf(TargetFile, "group = %d\n", nLastGroup);
				fprintf(TargetFile, "ports => %d-%d\n", nFirstPort, nLastPort);	        	
				fprintf(TargetFile, "\n");	    
	    	}	    
	    	break;			
		case DG_FXO_INTERFACE:
		    printf("Card Interface - Ports: FXO_INTERFACE - %d\n", nCardPortsCount);

		    nLastGroup++;

		    //write group
		    printf("[groups]\n");
	    	printf("group = %d\n", nLastGroup);
		    printf("ports => %d-%d\n", nFirstPort, nLastPort);
		    //printf("\n");
	
		    fprintf(TargetFile, "group = %d\n", nLastGroup);
		    fprintf(TargetFile, "ports => %d-%d\n", nFirstPort, nLastPort);	        	
	    	fprintf(TargetFile, "\n");    	    
		    break;
		case DG_FXS_INTERFACE:
		    printf("Card Interface - Ports: FXS_INTERFACE - %d\n", nCardPortsCount);

		    nLastGroup++;

		    //write group
		    printf("[groups]\n");
	    	printf("group = %d\n", nLastGroup);
		    printf("ports => %d-%d\n", nFirstPort, nLastPort);
		    //printf("\n");
	
		    fprintf(TargetFile, "group = %d\n", nLastGroup);
		    fprintf(TargetFile, "ports => %d-%d\n", nFirstPort, nLastPort);	        	
	    	fprintf(TargetFile, "\n");    	    
		    break;			
    }
    
    fclose(TargetFile);
}

void ConfigPorts(short nCard)
{
    FILE *TargetFile;    
    int nCardInterface, nCardPortsCount, nFirstPort, nLastPort;

    nCardInterface = dg_GetCardInterface(nCard);
    nCardPortsCount = dg_GetCardPortsCount(nCard);
    nFirstPort = dg_GetAbsolutePortNumber(nCard, 1); 
    nLastPort = dg_GetAbsolutePortNumber(nCard, nCardPortsCount); 

    if ((TargetFile = fopen("digivoice.conf", "a")) == NULL)
    {
        printf("\n\nError Reading TargetFile-ConfigPorts");
    }

    switch(nCardInterface)
    {
		case DG_DIGITAL_INTERFACE:
	    	printf("\nCard Interface - Ports: DIGITAL_INTERFACE - %d\n", nCardPortsCount);
	    
		    if (nCardPortsCount == 60)
		    {
	    	    //step_4 = default file for DIGITAL card type
	        	//printf("step_4/8...\n");
		        CopyFileToFile("step_4.conf", "digivoice.conf");
		
	    	    //write port_config
				printf("[port_config]\n");
				printf("ports => %d-%d\n", nFirstPort, nLastPort - 30);		
				printf("\n");
		
				fprintf(TargetFile, "ports => %d-%d\n", nFirstPort, nLastPort - 30);    	    		
				fprintf(TargetFile, "\n");		
		
				fclose(TargetFile);
		
	        	if ((TargetFile = fopen("digivoice.conf", "a")) == NULL)
	        	{
	            	printf("\n\nError Reading TargetFile-ConfigPorts");
				}		
	    
	        	//step_5 = default file for DIGITAL card type
	        	//printf("step_5/8...\n");
	        	CopyFileToFile("step_5.conf", "digivoice.conf");
		
	        	//write port_config
				printf("[port_config]\n");
				printf("ports => %d-%d\n", nLastPort - 29, nLastPort);
		
				fprintf(TargetFile, "ports => %d-%d\n", nLastPort - 29, nLastPort);	    
	    	}
	    	else
	    	{
	        	//step_4 = default file for DIGITAL card type
	        	//printf("step_4/8...\n");
	        	CopyFileToFile("step_4.conf", "digivoice.conf");
		
	        	//write port_config
				printf("[port_config]\n");
				printf("ports => %d-%d\n", nFirstPort, nLastPort);
		
				fprintf(TargetFile, "ports => %d-%d\n", nFirstPort, nLastPort);    			
	    	}	    
		    break;
		case DG_FXO_INTERFACE:
		    printf("\nCard Interface - Ports: FXO_INTERFACE - %d\n", nCardPortsCount);

		    //step_6 = default file for FXO card type
		    //printf("step_6/8...\n");
		    CopyFileToFile("step_6.conf", "digivoice.conf");
		
		    //write port_config
		    printf("[port_config]\n");
	    	printf("ports => %d-%d\n", nFirstPort, nLastPort);
		
	    	fprintf(TargetFile, "ports => %d-%d\n", nFirstPort, nLastPort);    
	    
	    	break;	    
		case DG_FXS_INTERFACE:
		    printf("\nCard Interface - Ports: FXS_INTERFACE - %d\n", nCardPortsCount);

		    //step_7 = default file for FXS card type
		    //printf("step_7/8...\n");
		    CopyFileToFile("step_7.conf", "digivoice.conf");
		
		    //write port_config
		    printf("[port_config]\n");
	    	printf("ports => %d-%d\n", nFirstPort, nLastPort);
		
	    	fprintf(TargetFile, "ports => %d-%d\n", nFirstPort, nLastPort);    
	    
	    	break;    
    }
	
    fclose(TargetFile);     
}

void ConfigE1(int nCardsCount)
{
    FILE *TargetFile;    
    int nForCardsCount;
    char szCards[MAX_PATH];
    char szTemp[MAX_PATH];    
    
    if ((TargetFile = fopen("digivoice.conf", "a")) == NULL)
    {
        printf("\n\nError Reading TargetFile-ConfigE1");
    }

    szCards[0] = '\0';    
    for (nForCardsCount=1; nForCardsCount<=nCardsCount; nForCardsCount++)
    {       
	if (dg_GetCardInterface(nForCardsCount) == DG_DIGITAL_INTERFACE)
	{
	    sprintf(szTemp, "%d,", nForCardsCount);
	    strcat(szCards, szTemp);	
	}	
    }	
	
    szCards[strlen(szCards) - 1] = '\0';
    
    CopyFileToFile("step_8.conf", "digivoice.conf");
    		
    //write e1_config
    printf("\n[e1_config]\n");
    printf("e1_sync=1\n");
    printf("use_crc4=0\n");
    printf("cards => %s\n", szCards);
		
    fprintf(TargetFile, "cards => %s\n", szCards);    
	    
    fclose(TargetFile);     
}

int main(int argc, char *argv[])
{
    int nRetStartVLib;
    int nCardsCount;
    int nForCardsCount;
    int nE1CardsCount;

    //Start VoicerLib
    if ((nRetStartVLib = dg_StartVoicerlib(NULL)) != DG_EXIT_SUCCESS)
    {
	printf("\n\nError Starting VoicerLib (code %d)\n", nRetStartVLib);
	exit(EXIT_FAILURE);
    }
    printf("\nVoicerLib Started!\n");    
	
    //CardsCount
    nCardsCount = dg_GetCardsCount();
    printf("\n%d Card(s) Detected...\n\n", nCardsCount);

    //step_1 = default file for any card type
    //printf("\n----- step 1/8 -----\n");
    CopyFileToFile("step_1.conf", "digivoice.conf");    
	
    //step_2 = config groups
    //printf("\n----- step 2/8 -----\n");        
    nLastGroup = 0;    
    for (nForCardsCount=1; nForCardsCount<=nCardsCount; nForCardsCount++)
    {
	//printf("Auto Group Configuration: %d\n", nForCardsCount);		
	ConfigGroups(nForCardsCount);
    }
	
    //step_3 = default file for any card type
    //printf("----- step 3/8 -----\n");    
    CopyFileToFile("step_3.conf", "digivoice.conf");      
    
    //step4-6 = config port_config
    //printf("\n----- step 4-6/8 -----");        
    for (nForCardsCount=1; nForCardsCount<=nCardsCount; nForCardsCount++)
    {
	//printf("\nAuto Port Configuration: %d\n", nForCardsCount);		
	ConfigPorts(nForCardsCount);
    }
    
    //step_8 = config e1_config
    nE1CardsCount = 0;
    for (nForCardsCount=1; nForCardsCount<=nCardsCount; nForCardsCount++)
    {
	if (dg_GetCardInterface(nForCardsCount) == DG_DIGITAL_INTERFACE)
	    nE1CardsCount++;
    }        
    if (nE1CardsCount >= 1)
    {
	//printf("\n----- step 8/8 -----");        	    
        //printf("\nAuto E1 Configuration: %d Card(s)\n", nE1CardsCount);		
	ConfigE1(nCardsCount);    
    }	

    dg_ShutdownVoicerlib();

    printf("\n\n* * * Goodbye! * * *\n\n");
    return DG_EXIT_SUCCESS;
}
